@@ -170,7 +170,7 @@
    display-trace ; for backwards compatibility describe-request describe-event describe-reply
    closed-display-p
    ;; extensions
-   open-default-display with-open-display
+   open-default-display with-open-display events-queued
    display-get-default display-resource-manager-string screen-resource-string
    *canonicalize-encoding*
    ;;; Only when using libXt:
@@ -1612,13 +1612,6 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 	      (or (x-type-error-type-string condition)
 		  (type-error-expected-type condition))))))
 
-(define-condition closed-display (x-error)
-  ((display :reader closed-display-display :initarg :display))
-  (:report
-    (lambda (condition stream)
-      (format stream "Attempt to use closed display ~s"
-	      (closed-display-display condition)))))
-
 (define-condition lookup-error (x-error)
   ((id :reader lookup-error-id :initarg :id)
    (display :reader lookup-error-display :initarg :display)
@@ -1754,10 +1747,6 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 
 (define-condition implementation-error (request-error) ())
 
-(define-condition connection-failure (x-error) ())
-
-(define-condition device-busy (x-error) ())
-
 ;; (define-condition server-disconnect (x-error) ())
 
 (pushnew "XLIB" custom:*system-package-list* :test #'string=)
@@ -1773,7 +1762,6 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 (undefined DRAW-GLYPH)
 (undefined DRAW-IMAGE-GLYPH)
 (undefined TRANSLATE-DEFAULT)
-(undefined QUEUE-EVENT)
 )
 
 ;; canonicalize encodings supplied by X, see clx.f:get_font_info_and_display()
