@@ -1,7 +1,7 @@
 files='dirkey.o'
-make clisp-module \
-     CC="${CC}" CPPFLAGS="${CPPFLAGS}" CFLAGS="${CFLAGS}" \
-     INCLUDES="$absolute_linkkitdir"
+${MAKE-make} clisp-module \
+  CC="${CC}" CPPFLAGS="${CPPFLAGS}" CFLAGS="${CFLAGS}" \
+  INCLUDES="$absolute_linkkitdir"
 NEW_FILES="${files}"
 NEW_LIBS="${files}"
 NEW_MODULES='dirkey'
