@@ -1,7 +1,7 @@
 /*
  * ======= General internationalization, for Lisp programs too =======
  * Copyright (C) 1990-2005 Bruno Haible
- * Copyright (C) 1998-2006 Sam Steingold
+ * Copyright (C) 1998-2008 Sam Steingold
  * GPL2
  */
 
@@ -30,8 +30,8 @@ DEFMODULE(i18n,"I18N")
 
 /* Returns the <locale.h> value corresponding to a LC_... constant. */
 DEFCHECKER(check_locale_category,prefix=LC,default=LC_MESSAGES, \
-           ALL COLLATE CTYPE MESSAGES MONETARY NUMERIC TIME \
-           PAPER NAME ADDRESS TELEPHONE MEASUREMENT IDENTIFICATION)
+           :ALL COLLATE CTYPE MESSAGES MONETARY NUMERIC TIME            \
+           PAPER :NAME ADDRESS TELEPHONE MEASUREMENT IDENTIFICATION)
 
 #ifdef GNU_GETTEXT
 
@@ -42,13 +42,13 @@ static inline object do_gettext (const char* msgid,
   if (msgid[0] == '\0') {
     translated_msg = "";  /* Don't return the catalog's header entry. */
   } else {
-    begin_system_call();
+    begin_blocking_system_call();
 #  ifdef CLISP_UNICODE
     if (domain != NULL)
       bind_textdomain_codeset(domain,"UTF-8");
 #  endif
     translated_msg = dcgettext(domain,msgid,category);
-    end_system_call();
+    end_blocking_system_call();
   }
   return asciz_to_string(translated_msg,Symbol_value(S(utf_8)));
 }
@@ -57,13 +57,13 @@ static inline object do_ngettext (const char* msgid, const char* msgid_plural,
                                   const char* domain, uint32 n, int category)
 {
   const char* translated_msg;
-  begin_system_call();
+  begin_blocking_system_call();
 # ifdef CLISP_UNICODE
   if (domain != NULL)
     bind_textdomain_codeset(domain,"UTF-8");
 # endif
   translated_msg = dcngettext(domain,msgid,msgid_plural,n,category);
-  end_system_call();
+  end_blocking_system_call();
   return asciz_to_string(translated_msg,Symbol_value(S(utf_8)));
 }
 
@@ -182,17 +182,18 @@ DEFUN(I18N:TEXTDOMAINDIR, domain)
 
 DEFUN(I18N:SET-TEXTDOMAINDIR, domain directory)
 { /* sets the message catalog directory for the given domain. */
-  object domain = (STACK_1=check_string(STACK_1));
+  STACK_1 = check_string(STACK_1);
  #ifdef GNU_GETTEXT
-  /* Check and use default directory, because the bindtextdomain()
-     documentation recommends that the argument be an absolute pathname,
-     to protect against later chdir() calls. */
-  object directory = pathname_to_OSdir(STACK_0,true);
-  with_string_0(STACK_1/*domain*/,Symbol_value(S(ascii)),domain_asciz, {
-    begin_system_call();
-    bindtextdomain(domain_asciz,TheAsciz(directory));
-    end_system_call();
-  });
+  { /* Check and use default directory, because the bindtextdomain()
+       documentation recommends that the argument be an absolute pathname,
+       to protect against later chdir() calls. */
+    object directory = pathname_to_OSdir(STACK_0,true);
+    with_string_0(STACK_1/*domain*/,Symbol_value(S(ascii)),domain_asciz, {
+        begin_system_call();
+        bindtextdomain(domain_asciz,TheAsciz(directory));
+        end_system_call();
+      });
+  }
  #endif
   VALUES1(STACK_0);
   skipSTACK(2);
@@ -200,7 +201,6 @@ DEFUN(I18N:SET-TEXTDOMAINDIR, domain directory)
 
 
 /* ======================== locale ======================== */
-
 DEFUN(I18N:SET-LOCALE, &optional category locale)
 { /* call setlocale(3) */
   gcv_object_t *category = &STACK_1;
@@ -214,7 +214,7 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
         res = setlocale(check_locale_category_map.table[pos].c_const,NULL);
         end_system_call();
         pushSTACK(*check_locale_category_map.table[pos].l_const);
-        pushSTACK(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
+        pushSTACK(safe_to_string(res));
       }
     } else {
       *locale = check_string(*locale);
@@ -224,7 +224,7 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
             res = setlocale(check_locale_category_map.table[pos].c_const,loc_z);
             end_system_call();
             pushSTACK(*check_locale_category_map.table[pos].l_const);
-            pushSTACK(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
+            pushSTACK(safe_to_string(res));
           }
         });
     }
@@ -243,7 +243,7 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
           end_system_call();
         });
     }
-    VALUES1(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
+    VALUES1(safe_to_string(res));
   }
   skipSTACK(2);
 }
@@ -260,7 +260,7 @@ static void get_locale_info (int what, char**res, int *res_size) {
   end_system_call();
   if (val == 0) OS_error();
   if (val > *res_size) {
-    *res = (char*)my_realloc(*res,val);
+    *res = (char*)clisp_realloc(*res,val);
     if (*res == NULL) OS_error();
     *res_size = val;
     begin_system_call();
@@ -385,7 +385,7 @@ static void locale_bool_to_STACK (int what, char**res, int* res_size) {
 DEFUN(I18N:LOCALE-CONV,)
 { /* call GetLocaleInfo(3) */
   int res_size = GET_LOCALE_INFO_BUF_SIZE;
-  char *res = my_malloc(res_size);
+  char *res = clisp_malloc(res_size);
   locale_string_to_STACK(LOCALE_SDECIMAL,&res,&res_size);
   locale_string_to_STACK(LOCALE_STHOUSAND,&res,&res_size);
   thousands_sep_to_STACK(LOCALE_SGROUPING,&res,&res_size);
@@ -474,13 +474,13 @@ DEFCHECKER(check_nl_item,CODESET                                        \
 #if defined(HAVE_NL_LANGINFO)
 # define get_lang_info(what)                                            \
   begin_system_call(); res = nl_langinfo(what); end_system_call()
-# define res_to_obj() (res ? asciz_to_string(res,GLO(misc_encoding)) : NIL)
+# define res_to_obj() safe_to_string(res)
 # define DECLARE_RES  char* res
 # define FINISH_RES
 #elif defined(WIN32_NATIVE)
 # define get_lang_info(what)  get_locale_info(what,&res,&res_size)
-# define res_to_obj() (asciz_to_string(res,GLO(misc_encoding)))
-# define DECLARE_RES  int res_size=GET_LOCALE_INFO_BUF_SIZE; char *res=(char*)my_malloc(res_size)
+# define res_to_obj() asciz_to_string(res,GLO(misc_encoding))
+# define DECLARE_RES  int res_size=GET_LOCALE_INFO_BUF_SIZE; char *res=(char*)clisp_malloc(res_size)
 # define FINISH_RES   begin_system_call(); free(res); end_system_call()
 #endif
 DEFUNR(I18N:LANGUAGE-INFORMATION,&optional item)
