@@ -1,5 +1,5 @@
 ;; CLISP interface to PostgreSQL <http://www.postgresql.org/>
-;; Copyright (C) 1999-2008 Sam Steingold
+;; Copyright (C) 1999-2009 Sam Steingold
 ;; This is free software, distributed under the GNU GPL 2
 
 ;; this file is in sync with libpq-fe.h 1.141
@@ -26,7 +26,7 @@
 (ffi:default-foreign-language :stdc)
 (eval-when (compile) (setq *foreign-guard* t))
 
-(c-lines "#include <config.h>~%") ; local PostgreSQL config
+(c-lines "#include \"config.h\"~%") ; local PostgreSQL config
 
 (c-lines "#if defined(HAVE_POSTGRES_EXT_H)
 #  include <postgres_ext.h>
@@ -279,21 +279,19 @@
   (:arguments (conn PGconn) (query c-string)))
 (def-call-out PQexecParams (:return-type PGresult)
   (:arguments (conn PGconn) (command c-string) (nParams int)
-              (paramTypes (c-ptr (c-array-max Oid #.MAX-PARAM)) :out :alloca)
-              (paramValues (c-ptr (c-array-max c-string #.MAX-PARAM))
-                           :out :alloca)
-              (paramLengths (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
-              (paramFormats (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
+              (paramTypes (c-array-ptr Oid)) ; at least nParams
+              (paramValues (c-array-ptr c-string)) ; ditto
+              (paramLengths (c-array-ptr int))     ; ditto
+              (paramFormats (c-array-ptr int))     ; ditto
               (resultFormat int)))
 (def-call-out PQprepare (:return-type PGresult)
   (:arguments (conn PGconn) (stmtName c-string) (query c-string) (nParams int)
-              (paramTypes (c-ptr (c-array-max Oid #.MAX-PARAM)) :out :alloca)))
+              (paramTypes (c-array-ptr Oid)))) ; at least nParams
 (def-call-out PQexecPrepared (:return-type PGresult)
   (:arguments (conn PGconn) (stmtName c-string) (nParams int)
-              (paramValues (c-ptr (c-array-max c-string #.MAX-PARAM))
-                           :out :alloca)
-              (paramLengths (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
-              (paramFormats (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
+              (paramValues (c-array-ptr c-string)) ; at least nParams
+              (paramLengths (c-array-ptr int))     ; ditto
+              (paramFormats (c-array-ptr int))     ; ditto
               (resultFormat int)))
 
 ;; Interface for multiple-result or asynchronous queries
@@ -301,21 +299,19 @@
   (:arguments (conn PGconn) (query c-string)))
 (def-call-out PQsendQueryParams (:return-type int)
   (:arguments (conn PGconn) (command c-string) (nParams int)
-              (paramTypes (c-ptr (c-array-max Oid #.MAX-PARAM)) :out :alloca)
-              (paramValues (c-ptr (c-array-max c-string #.MAX-PARAM))
-                           :out :alloca)
-              (paramLengths (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
-              (paramFormats (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
+              (paramTypes (c-array-ptr Oid)) ; at least nParams
+              (paramValues (c-array-ptr c-string)) ; ditto
+              (paramLengths (c-array-ptr int))     ; ditto
+              (paramFormats (c-array-ptr int))     ; ditto
               (resultFormat int)))
 (def-call-out PQsendPrepare (:return-type int)
   (:arguments (conn PGconn) (stmtName c-string) (query c-string) (nParams int)
-              (paramTypes (c-ptr (c-array-max Oid #.MAX-PARAM)) :out :alloca)))
+              (paramTypes (c-array-ptr Oid)))) ; at least nParams
 (def-call-out PQsendQueryPrepared (:return-type int)
   (:arguments (conn PGconn) (stmtName c-string) (nParams int)
-              (paramValues (c-ptr (c-array-max c-string #.MAX-PARAM))
-                           :out :alloca)
-              (paramLengths (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
-              (paramFormats (c-ptr (c-array-max int #.MAX-PARAM)) :out :alloca)
+              (paramValues (c-array-ptr c-string)) ; at least nParams
+              (paramLengths (c-array-ptr int))     ; ditto
+              (paramFormats (c-array-ptr int))     ; ditto
               (resultFormat int)))
 (def-call-out PQgetResult (:return-type PGresult) (:arguments (conn PGconn)))
 
@@ -359,7 +355,7 @@
 (def-call-out PQfn (:return-type PGresult)
   (:arguments (conn PGconn) (fnid int) (result_buf (c-ptr int) :out)
               (result_len (c-ptr int) :out) (result_is_int int)
-              (args (c-ptr (c-array-max PQArgBlock #.MAX-PARAM)) :out :alloca)
+              (args (c-array-ptr PQArgBlock)) ; at least nargs
               (nargs int)))
 
 ;; Accessor functions for PGresult objects
@@ -434,7 +430,8 @@
               (from c-string) (length size_t) (error (c-ptr int) :out)))
 (def-call-out PQescapeByteaConn (:return-type (c-array-max uchar #.BUFSIZ))
   (:arguments (conn PGconn)
-              (from (c-array-max uchar #.BUFSIZ)) (from_length size_t)
+              (from (c-array-ptr uchar)) ; at least from_length
+              (from_length size_t)
               (to_length (c-ptr size_t) :out)))
 (def-call-out PQunescapeBytea (:return-type (c-array-max uchar #.BUFSIZ))
   (:arguments (strtext (c-ptr (c-array-max uchar #.BUFSIZ)) :in-out)
