@@ -2,9 +2,9 @@
  * Module for Raw Sockets / CLISP
  * Fred Cohen, 2003-2004
  * Don Cohen, 2003-2004
- * Sam Steingold 2004-2008
+ * Sam Steingold 2004-2009
  * Bruno Haible 2004-2008
- * <http://www.opengroup.org/onlinepubs/007908799/xns/syssocket.h.html>
+ * <http://www.opengroup.org/onlinepubs/9699919799/basedefs/sys_socket.h.html>
  */
 
 #if defined(_WIN32)
@@ -93,8 +93,8 @@ typedef SOCKET rawsock_t;
 DEFMODULE(rawsock,"RAWSOCK")
 
 /* ================== helpers ================== */
-/* DANGER: the return value is invalidated by GC!
- > *arg_: vector
+/* DANGER: the return value is invalidated by GC! only used by with_buffer_arg
+ <> *arg_: vector; on return: simple byte vector, to be pinned
  > STACK_0, STACK_1: END & START -- removed
  > prot: PROT_READ or PROT_READ_WRITE
  < size: how many bytes to use
@@ -105,7 +105,7 @@ static void* parse_buffer_arg (gcv_object_t *arg_, size_t *size, int prot) {
   stringarg sa;
   *arg_ = check_byte_vector(*arg_);
   sa.offset = 0; sa.len = vector_length(*arg_);
-  sa.string = array_displace_check(*arg_,sa.len,&sa.offset);
+  sa.string = *arg_ = array_displace_check(*arg_,sa.len,&sa.offset);
   test_vector_limits(&sa);
   *size = sa.len;
   { void *start_address =
@@ -114,6 +114,13 @@ static void* parse_buffer_arg (gcv_object_t *arg_, size_t *size, int prot) {
     return start_address;
   }
 }
+#define with_buffer_arg(_buffer,_arg_,_size,_prot,_code)  do {  \
+    void *_buffer = parse_buffer_arg(_arg_,_size,_prot);        \
+    pin_varobject(*_arg_);                                      \
+    _code;                                                      \
+    unpin_varobject(*_arg_);                                    \
+  } while(0)
+
 /* DANGER: the return value is invalidated by GC!
  > type: the expected type
  > arg: the argument
@@ -199,7 +206,8 @@ static void fill_iovec (object vect, size_t offset, ssize_t veclen,
 DEFUN(RAWSOCK:SOCKADDR-FAMILY, sa) {
   CLISP_SOCKLEN_T size;
   struct sockaddr *sa = CHECK_SOCKADDR(&STACK_0,&size,PROT_READ);
-  VALUES2(fixnum(sa->sa_family),fixnum(size)); skipSTACK(1);
+  VALUES2(check_socket_domain_reverse(sa->sa_family),fixnum(size));
+  skipSTACK(1);
 }
 DEFUN(RAWSOCK::SOCKADDR-SLOT,&optional slot) {
   /* return offset & size of the slot in SOCKADDR */
@@ -218,7 +226,7 @@ DEFUN(RAWSOCK::SOCKADDR-SLOT,&optional slot) {
     pushSTACK(NIL);             /* no PLACE */
     pushSTACK(STACK_1/*slot*/); /* TYPE-ERROR slot DATUM */
     pushSTACK(`(MEMBER :FAMILY :DATA)`); /* TYPE-ERROR slot EXPECTED-TYPE */
-    pushSTACK(`SOCKADDR`); pushSTACK(STACK_2/*slot*/);
+    pushSTACK(`RAWSOCK::SOCKADDR`); pushSTACK(STACK_2/*slot*/);
     pushSTACK(TheSubr(subr_self)->name);
     check_value(type_error,GETTEXT("~S: unknown slot ~S for ~S"));
     STACK_0 = value1;
@@ -283,22 +291,31 @@ DEFUN(RAWSOCK:MAKE-SOCKADDR,family &optional data) {
 }
 
 /* ================== SIGPIPE handling ================== */
-#if defined(HAVE_SIGNALS)
-extern bool writing_to_subprocess;
-# define begin_sock_call()  writing_to_subprocess=true;  begin_system_call()
-# define end_sock_call()    writing_to_subprocess=false; end_system_call()
-#else
-# define begin_sock_call()  begin_system_call()
-# define end_sock_call()    end_system_call()
-#endif
+#define begin_sock_call()  START_WRITING_TO_SUBPROCESS;  begin_blocking_system_call()
+#define end_sock_call()    end_blocking_system_call(); STOP_WRITING_TO_SUBPROCESS
+
+/* report error on the given socket or OS_error if socket<0 */
+nonreturning_function(static, rawsock_error, (int socket)) {
+  if (socket < 0) OS_error();
+  begin_system_call(); {
+    int ecode = errno;
+    char *msg = strerror(ecode);
+    end_system_call();
+    pushSTACK(`RAWSOCK::RAWSOCK-ERROR`);    /* error type */
+    pushSTACK(S(Kcode));
+    pushSTACK(fixnum(errno)); funcall(`OS::ERRNO`,1); pushSTACK(value1);
+    pushSTACK(`:MESSAGE`); pushSTACK(safe_to_string(msg));
+    pushSTACK(`:SOCKET`); pushSTACK(fixnum(socket));
+    funcall(S(make_instance),7);
+  }
+  pushSTACK(value1); funcall(S(error),1);
+  NOTREACHED;
+}
 
 /* invoke system call C, place return value in R, report error on socket S */
-#define SYSCALL(r,s,c)                                  \
-  do { begin_sock_call(); r = c; end_sock_call();       \
-    if (r == -1) {                                      \
-      if (s<=0) OS_error();                             \
-      else OS_file_error(fixnum(s));                    \
-    }                                                   \
+#define SYSCALL(r,s,c)                                                  \
+  do { begin_sock_call(); r = c; end_sock_call();                       \
+    if (r == -1) rawsock_error(s);                                      \
   } while(0)
 
 /* ================== arpa/inet.h interface ================== */
@@ -417,7 +434,7 @@ static Values protoent_to_protocol (struct protoent *pe) {
 }
 DEFUN(RAWSOCK:PROTOCOL, &optional protocol)
 { /* interface to getprotobyname() et al
-     http://www.opengroup.org/onlinepubs/009695399/functions/getprotoent.html */
+     http://www.opengroup.org/onlinepubs/9699919799/functions/getprotoent.html */
   object proto = popSTACK();
   struct protoent *pe = NULL;
   if (missingp(proto)) {        /* get all protocols */
@@ -464,7 +481,7 @@ static Values netent_to_network (struct netent *ne) {
 }
 DEFUN(RAWSOCK:NETWORK, &optional network type)
 { /* interface to getnetbyname() et al
-     http://www.opengroup.org/onlinepubs/009695399/functions/getnetent.html */
+     http://www.opengroup.org/onlinepubs/9699919799/functions/getnetent.html */
   unsigned int type = check_uint_defaulted(popSTACK(),(unsigned int)-1);
   object net = popSTACK();
   struct netent *ne = NULL;
@@ -505,7 +522,7 @@ DEFUN(RAWSOCK:NETWORK, &optional network type)
 }
 #endif  /* HAVE_NETDB_H */
 /* ================== net/if.h interface ================== */
-/* http://www.opengroup.org/onlinepubs/009695399/basedefs/net/if.h.html */
+/* http://www.opengroup.org/onlinepubs/9699919799/basedefs/net_if.h.html */
 #if defined(HAVE_NET_IF_H)
 DEFUN(RAWSOCK:IF-NAME-INDEX, &optional what) {
   if (missingp(STACK_0)) {
@@ -559,7 +576,7 @@ DEFCHECKER(check_iff,prefix=IFF,bitmasks=both,default=(unsigned)~0,     \
            OACTIVE SIMPLEX LINK0 LINK1 LINK2 ALTPHYS POLLING PPROMISC   \
            MONITOR STATICARP NEEDSGIANT                                 \
            MASTER SLAVE MULTICAST PORTSEL AUTOMEDIA DYNAMIC LOWER-UP DORMANT)
-DEFUN(RAWSOCK:IFADDRS,&key flags-and flags-or) {
+DEFUN(RAWSOCK:IFADDRS,&key FLAGS-AND FLAGS-OR) {
   struct ifaddrs *ifap;
   int count = 0;
   unsigned int flags_or = check_iff_of_list(popSTACK());
@@ -794,7 +811,7 @@ nonreturning_function(static, error_eai, (int ecode)) {
   end_system_call();
   pushSTACK(`RAWSOCK::EAI`);    /* error type */
   pushSTACK(S(Kcode)); pushSTACK(check_gai_ecode_reverse(ecode));
-  pushSTACK(`:MESSAGE`); pushSTACK(asciz_to_string(msg,GLO(misc_encoding)));
+  pushSTACK(`:MESSAGE`); pushSTACK(safe_to_string(msg));
   funcall(S(make_instance),5);
   pushSTACK(value1); funcall(S(error),1);
   NOTREACHED;
@@ -913,8 +930,9 @@ DEFUN(RAWSOCK:RECV,socket buffer &key :START :END PEEK OOB WAITALL) {
   rawsock_t sock = I_to_uint(check_uint(STACK_3));
   int retval;
   size_t buffer_len;
-  void *buffer = parse_buffer_arg(&STACK_2,&buffer_len,PROT_READ_WRITE);
-  SYSCALL(retval,sock,recv(sock,(BUF_TYPE_T)buffer,buffer_len,flags));
+  with_buffer_arg(buffer,&STACK_2,&buffer_len,PROT_READ_WRITE,
+                  SYSCALL(retval,sock,recv(sock,(BUF_TYPE_T)buffer,
+                                           buffer_len,flags)));
   VALUES1(fixnum(retval)); skipSTACK(2);
 }
 
@@ -930,10 +948,9 @@ DEFUN(RAWSOCK:RECVFROM,socket buffer address &key :START :END PEEK OOB WAITALL){
   if (!missingp(STACK_1)) STACK_1 = check_posfixnum(STACK_1);
   STACK_3 = check_byte_vector(STACK_3);
   optional_sockaddr_argument(&STACK_2,&sa,&sa_size);
-  /* no GC after this point! - buffer, start, end have been checked already */
-  buffer = parse_buffer_arg(&STACK_3,&buffer_len,PROT_READ_WRITE);
-  SYSCALL(retval,sock,recvfrom(sock,(BUF_TYPE_T)buffer,
-                               buffer_len,flags,sa,&sa_size));
+  with_buffer_arg(buffer,&STACK_3,&buffer_len,PROT_READ_WRITE,
+                  SYSCALL(retval,sock,recvfrom(sock,(BUF_TYPE_T)buffer,
+                                               buffer_len,flags,sa,&sa_size)));
   VALUES3(fixnum(retval),fixnum(sa_size),STACK_0); skipSTACK(3);
 }
 
@@ -1001,8 +1018,8 @@ DEFUN(RAWSOCK:RECVMSG,socket message &key :START :END PEEK OOB WAITALL) {
 #endif  /* HAVE_RECVMSG & HAVE_MSGHDR_MSG_FLAGS & HAVE_MSGHDR_MSG_CONTROL */
 
 DEFUN(RAWSOCK:SOCK-READ,socket buffer &key :START :END)
-{ /* http://www.opengroup.org/onlinepubs/009695399/functions/read.html
-     http://www.opengroup.org/onlinepubs/009695399/functions/readv.html */
+{ /* http://www.opengroup.org/onlinepubs/9699919799/functions/read.html
+     http://www.opengroup.org/onlinepubs/9699919799/functions/readv.html */
   rawsock_t sock = I_to_uint(check_uint(STACK_3));
   ssize_t retval;
   size_t len;
@@ -1011,10 +1028,9 @@ DEFUN(RAWSOCK:SOCK-READ,socket buffer &key :START :END)
     struct iovec *buffer = (struct iovec*)alloca(sizeof(struct iovec)*retval);
     fill_iovec(STACK_0,offset,retval,buffer,PROT_READ_WRITE);
     SYSCALL(retval,sock,readv(sock,buffer,retval));
-  } else {                      /* READ */
-    void *buffer = parse_buffer_arg(&STACK_2,&len,PROT_READ_WRITE);
-    SYSCALL(retval,sock,READ(sock,buffer,len));
-  }
+  } else                       /* READ */
+    with_buffer_arg(buffer,&STACK_2,&len,PROT_READ_WRITE,
+                    SYSCALL(retval,sock,READ(sock,buffer,len)));
   VALUES1(ssize_to_I(retval)); skipSTACK(2);
 }
 
@@ -1028,8 +1044,9 @@ DEFUN(RAWSOCK:SEND,socket buffer &key :START :END OOB EOR) {
   rawsock_t sock = I_to_uint(check_uint(STACK_3));
   int retval;
   size_t buffer_len;
-  void *buffer = parse_buffer_arg(&STACK_2,&buffer_len,PROT_READ);
-  SYSCALL(retval,sock,send(sock,(const BUF_TYPE_T)buffer,buffer_len,flags));
+  with_buffer_arg(buffer,&STACK_2,&buffer_len,PROT_READ,
+                  SYSCALL(retval,sock,send(sock,(const BUF_TYPE_T)buffer,
+                                           buffer_len,flags)));
   VALUES1(fixnum(retval)); skipSTACK(2);
 }
 
@@ -1057,23 +1074,21 @@ DEFUN(RAWSOCK:SENDTO, socket buffer address &key :START :END OOB EOR) {
   rawsock_t sock = I_to_uint(check_uint(STACK_4));
   int retval;
   struct sockaddr *sa;
-  void *buffer;
   size_t buffer_len;
   CLISP_SOCKLEN_T size;
   if (!missingp(STACK_0)) STACK_0 = check_posfixnum(STACK_0);
   if (!missingp(STACK_1)) STACK_1 = check_posfixnum(STACK_1);
   STACK_3 = check_byte_vector(STACK_3);
   sa = CHECK_SOCKADDR(&STACK_2,&size,PROT_READ);
-  /* no GC after this point! - buffer, start, end have been checked already */
-  buffer = parse_buffer_arg(&STACK_3,&buffer_len,PROT_READ);
-  SYSCALL(retval,sock,sendto(sock,(const BUF_TYPE_T)buffer,
-                             buffer_len,flags,sa,size));
+  with_buffer_arg(buffer,&STACK_3,&buffer_len,PROT_READ,
+                  SYSCALL(retval,sock,sendto(sock,(const BUF_TYPE_T)buffer,
+                                             buffer_len,flags,sa,size)));
   VALUES1(fixnum(retval)); skipSTACK(3);
 }
 
 DEFUN(RAWSOCK:SOCK-WRITE,socket buffer &key :START :END)
-{ /* http://www.opengroup.org/onlinepubs/009695399/functions/write.html
-     http://www.opengroup.org/onlinepubs/009695399/functions/writev.html */
+{ /* http://www.opengroup.org/onlinepubs/9699919799/functions/write.html
+     http://www.opengroup.org/onlinepubs/9699919799/functions/writev.html */
   rawsock_t sock = I_to_uint(check_uint(STACK_3));
   ssize_t retval;
   size_t len;
@@ -1082,10 +1097,9 @@ DEFUN(RAWSOCK:SOCK-WRITE,socket buffer &key :START :END)
     struct iovec *buffer = (struct iovec*)alloca(sizeof(struct iovec)*retval);
     fill_iovec(STACK_0,offset,retval,buffer,PROT_READ);
     SYSCALL(retval,sock,writev(sock,buffer,retval));
-  } else {                      /* WRITE */
-    void *buffer = parse_buffer_arg(&STACK_2,&len,PROT_READ);
-    SYSCALL(retval,sock,WRITE(sock,buffer,len));
-  }
+  } else                        /* WRITE */
+    with_buffer_arg(buffer,&STACK_2,&len,PROT_READ,
+                    SYSCALL(retval,sock,WRITE(sock,buffer,len)));
   VALUES1(ssize_to_I(retval)); skipSTACK(2);
 }
 
@@ -1134,9 +1148,9 @@ DEFUN(RAWSOCK:CONFIGDEV, socket name ipaddress &key PROMISC NOARP) {
   uint32 ipaddress = I_to_UL(check_uint32(STACK_0));
   rawsock_t sock = I_to_uint(check_uint(STACK_2));
   with_string_0(check_string(STACK_1),Symbol_value(S(utf_8)),name, {
-      begin_system_call();
+      begin_blocking_system_call();
       configdev(sock, name, ipaddress, flags);
-      end_system_call();
+      end_blocking_system_call();
     });
   VALUES0; skipSTACK(3);
 }
@@ -1234,7 +1248,7 @@ static object get_sock_opt (rawsock_t sock, int level, int name, int err_p) {
 }
 #undef GET_SOCK_OPT
 DEFUN(RAWSOCK:SOCKET-OPTION, sock name &key :LEVEL)
-{ /* http://www.opengroup.org/onlinepubs/009695399/functions/getsockopt.html */
+{ /* http://www.opengroup.org/onlinepubs/9699919799/functions/getsockopt.html */
   int level = sockopt_level(popSTACK());
   int name = sockopt_name(popSTACK());
   rawsock_t sock;
@@ -1369,7 +1383,7 @@ static void set_sock_opt_many (rawsock_t sock, int level, int name,
 }
 
 DEFUN(RAWSOCK::SET-SOCKET-OPTION, value sock name &key :LEVEL)
-{ /* http://www.opengroup.org/onlinepubs/009695399/functions/setsockopt.html */
+{ /* http://www.opengroup.org/onlinepubs/9699919799/functions/setsockopt.html */
   int level = sockopt_level(popSTACK());
   int name = sockopt_name(popSTACK());
   rawsock_t sock;
@@ -1391,15 +1405,12 @@ DEFUN(RAWSOCK::SET-SOCKET-OPTION, value sock name &key :LEVEL)
 #endif
 
 /* ================== CHECKSUM from Fred Cohen ================== */
-DEFUN(RAWSOCK:IPCSUM, buffer &key :START :END) { /* IP checksum */
-  size_t length;
-  unsigned char* buffer =
-    (unsigned char*)parse_buffer_arg(&STACK_2,&length,PROT_READ_WRITE);
-  register long sum=0;           /* assumes long == 32 bits */
+static unsigned short ipcsum (unsigned char* buffer, size_t length) {
+  register long sum=0;          /* assumes long == 32 bits */
   unsigned short result;
   unsigned char *ptr=&(buffer[14]);
   unsigned int nbytes;
-  /* FIXME: check length > ??? */
+  ASSERT(length >= 26);         /* FIXME: is this right?! */
   buffer[24]=0;buffer[25]=0;nbytes=(buffer[14] & 0xF) << 2; /* checksum=0, headerlen */
   while(nbytes>1){sum += *ptr; ptr++; sum += *ptr <<8; ptr++; nbytes -= 2;}
   if(nbytes==1){sum += *ptr;}     /* mop up an odd byte,  if necessary */
@@ -1407,19 +1418,21 @@ DEFUN(RAWSOCK:IPCSUM, buffer &key :START :END) { /* IP checksum */
   result=~(sum  + (sum >> 16)) & 0xFFFF;
   buffer[24]=(result & 0xFF);
   buffer[25]=((result >> 8) & 0xFF);
-  VALUES1(fixnum(result));
-  skipSTACK(1);
+  return result;
+}
+DEFUN(RAWSOCK:IPCSUM, buffer &key :START :END) { /* IP checksum */
+  size_t length;
+  with_buffer_arg(buffer,&STACK_2,&length,PROT_READ_WRITE,
+                  length = ipcsum((unsigned char*)buffer,length));
+  VALUES1(fixnum(length)); skipSTACK(1);
 }
 
-DEFUN(RAWSOCK:ICMPCSUM, buffer &key :START :END) { /* ICMP checksum */
-  size_t length;
-  unsigned char* buffer =
-    (unsigned char*)parse_buffer_arg(&STACK_2,&length,PROT_READ);
-  register long sum=0;           /* assumes long == 32 bits */
+static unsigned short icmpcsum (unsigned char* buffer, size_t length) {
+  register long sum=0;          /* assumes long == 32 bits */
   unsigned short result;
   unsigned char *ptr;
   unsigned int nbytes, off, offset;
-  /* FIXME: check length > ??? */
+  ASSERT(length >= 18);                    /* FIXME: is this right?! */
   off=((buffer[14]&0xF)<<2);offset=off+14; /* start of ICMP header */
   buffer[offset+2]=0;buffer[offset+3]=0;
   nbytes=(((buffer[16])<<8)+(buffer[17]))-off; /* bytes in ICMP part */
@@ -1430,21 +1443,23 @@ DEFUN(RAWSOCK:ICMPCSUM, buffer &key :START :END) { /* ICMP checksum */
   result=~(sum  + (sum >> 16)) & 0xFFFF;
   buffer[offset+2]=(result & 0xFF);
   buffer[offset+3]=((result >> 8) & 0xFF);
-  VALUES1(fixnum(result));
-  skipSTACK(1);
+  return result;
+}
+DEFUN(RAWSOCK:ICMPCSUM, buffer &key :START :END) { /* ICMP checksum */
+  size_t length;
+  with_buffer_arg(buffer,&STACK_2,&length,PROT_READ,
+                  length = icmpcsum((unsigned char*)buffer,length));
+  VALUES1(fixnum(length)); skipSTACK(1);
 }
 
-DEFUN(RAWSOCK:TCPCSUM, buffer &key :START :END) { /* TCP checksum */
-  size_t length;
-  unsigned char* buffer =
-    (unsigned char*)parse_buffer_arg(&STACK_2,&length,PROT_READ_WRITE);
+static unsigned short tcpcsum (unsigned char* buffer, size_t length) {
   register unsigned long sum;  /* assumes long == 32 bits */
   unsigned short result;
   unsigned char *ptr;
   unsigned int nbytes, packsize, offset;
-  /* FIXME: check length > ??? */
-  sum = (buffer[26]<<8)+ buffer[27]+(buffer[28]<<8)+ buffer[29];  /* Src IP */
-  sum +=(buffer[30]<<8)+ buffer[31]+(buffer[32]<<8)+ buffer[33];  /* Dst IP */
+  ASSERT(length >= 34);         /* FIXME: is this right?! */
+  sum = (buffer[26]<<8)+ buffer[27]+(buffer[28]<<8)+ buffer[29]; /* Src IP */
+  sum +=(buffer[30]<<8)+ buffer[31]+(buffer[32]<<8)+ buffer[33]; /* Dst IP */
   sum +=(buffer[23]);           /* zero followed by protocol */
   packsize=((buffer[16])<<8)+(buffer[17]); /* packet size - not including ARP area */
   offset=((buffer[14]&0xF)<<2); /* start of TCP header (rel to IP header) */
@@ -1459,21 +1474,23 @@ DEFUN(RAWSOCK:TCPCSUM, buffer &key :START :END) { /* TCP checksum */
   result=~(sum  + (sum >> 16)) & 0xFFFF;
   buffer[offset+17+14]=(result & 0xFF);
   buffer[offset+16+14]=((result >> 8) & 0xFF);
-  VALUES1(fixnum(result));
-  skipSTACK(1);
+  return result;
+}
+DEFUN(RAWSOCK:TCPCSUM, buffer &key :START :END) { /* TCP checksum */
+  size_t length;
+  with_buffer_arg(buffer,&STACK_2,&length,PROT_READ_WRITE,
+                  length = tcpcsum((unsigned char*)buffer,length));
+  VALUES1(fixnum(length)); skipSTACK(1);
 }
 
-DEFUN(RAWSOCK:UDPCSUM, buffer &key :START :END) { /* UDP checksum */
-  size_t length;
-  unsigned char* buffer =
-    (unsigned char*)parse_buffer_arg(&STACK_2,&length,PROT_READ_WRITE);
+static unsigned short udpcsum (unsigned char* buffer, size_t length) {
   register unsigned long sum = 0;  /* assumes long == 32 bits */
   unsigned short result;
   unsigned char *ptr;
   unsigned int nbytes, packsize, offset;
-  /* FIXME: check length > ??? */
-  sum = (buffer[26]<<8)+ buffer[27]+(buffer[28]<<8)+ buffer[29];  /* Src IP */
-  sum +=(buffer[30]<<8)+ buffer[31]+(buffer[32]<<8)+ buffer[33];  /* Dst IP */
+  ASSERT(length >= 34);         /* FIXME: is this right?! */
+  sum = (buffer[26]<<8)+ buffer[27]+(buffer[28]<<8)+ buffer[29]; /* Src IP */
+  sum +=(buffer[30]<<8)+ buffer[31]+(buffer[32]<<8)+ buffer[33]; /* Dst IP */
   sum +=(buffer[23]);           /* zero followed by protocol */
   packsize=((buffer[16])<<8)+(buffer[17]); /* packet size */
   offset=((buffer[14]&0xF)<<2);            /* start of UDP header */
@@ -1488,8 +1505,13 @@ DEFUN(RAWSOCK:UDPCSUM, buffer &key :START :END) { /* UDP checksum */
   result=~(sum  + (sum >> 16)) & 0xFFFF;
   buffer[offset+7+14]=(result & 0xFF);
   buffer[offset+6+14]=((result >> 8) & 0xFF);
-  VALUES1(fixnum(result));
-  skipSTACK(1);
+  return result;
+}
+DEFUN(RAWSOCK:UDPCSUM, buffer &key :START :END) { /* UDP checksum */
+  size_t length;
+  with_buffer_arg(buffer,&STACK_2,&length,PROT_READ_WRITE,
+                  length = udpcsum((unsigned char*)buffer,length));
+  VALUES1(fixnum(length)); skipSTACK(1);
 }
 
 void module__rawsock__init_function_2 (module_t* module);
