@@ -30,7 +30,7 @@ DEFUN(REGEXP::REGEXP-COMPILE, pattern &key EXTENDED IGNORE-CASE NEWLINE NOSUB)
   int status;
   regex_t* re;
  restart_regcomp:
-  re = (regex_t*)my_malloc(sizeof(regex_t));
+  re = (regex_t*)clisp_malloc(sizeof(regex_t));
   with_string_0(pattern,GLO(misc_encoding),patternz, {
     begin_system_call();
     status = regcomp(re,patternz,cflags);
@@ -69,19 +69,25 @@ DEFUN(REGEXP::REGEXP-FREE, compiled)
   } else VALUES1(NIL);
 }
 
+typedef enum { ret_values, ret_list, ret_vector } rettype_t;
+#define CHECK_RETTYPE(x)                                                \
+  (eq(x,S(list)) ? ret_list : eq(x,S(vector)) ? ret_vector : ret_values)
+
 DEFFLAGSET(regexp_exec_flags, REG_NOTBOL REG_NOTEOL)
-DEFUN(REGEXP::REGEXP-EXEC,pattern string &key BOOLEAN :START :END NOTBOL NOTEOL)
+DEFUN(REGEXP::REGEXP-EXEC,pattern string &key           \
+      RETURN-TYPE BOOLEAN :START :END NOTBOL NOTEOL)
 { /* match the compiled pattern against the string */
   int eflags = regexp_exec_flags();
-  object string = (STACK_3 = check_string(STACK_3));
+  object string = (STACK_4 = check_string(STACK_4));
   unsigned int length = vector_length(string);
   unsigned int start = check_uint_defaulted(STACK_1,0);
   unsigned int end = check_uint_defaulted(STACK_0,length);
   int status;
   bool bool_p = !missingp(STACK_2);
+  rettype_t rettype = CHECK_RETTYPE(STACK_3);
   regex_t *re;
   regmatch_t *ret;
-  skipSTACK(3);                 /* drop all options */
+  skipSTACK(4);                 /* drop all options */
   for (;;) {
     STACK_1 = check_fpointer(STACK_1,true);
     re = (regex_t*)TheFpointer(STACK_1)->fp_pointer;
@@ -109,19 +115,35 @@ DEFUN(REGEXP::REGEXP-EXEC,pattern string &key BOOLEAN :START :END NOTBOL NOTEOL)
     status = regexec(re,stringz,re->re_nsub+1,ret,eflags);
     end_system_call();
   });
-  if (status) {
-    VALUES0;
-  } else if (bool_p) {
-    VALUES1(T);                 /* success indicator */
+  if (bool_p) {
+    VALUES_IF(!status);         /* success indicator */
+  } else if (status) {
+    switch (rettype) {
+      case ret_values: VALUES0; break;        /* VALUES => no values */
+      case ret_list:   VALUES1(NIL); break;   /* LIST => () */
+      case ret_vector: VALUES1(`#()`); break; /* VECTOR => #() */
+    }
   } else {
-    int count;
-    for (count = 0; count <= re->re_nsub; count++)
-      if (ret[count].rm_so >= 0 && ret[count].rm_eo >= 0) {
-        pushSTACK(posfixnum(start+ret[count].rm_so));
-        pushSTACK(posfixnum(start+ret[count].rm_eo));
+    uintL re_count;
+    for (re_count = 0; re_count <= re->re_nsub; re_count++)
+      if (ret[re_count].rm_so >= 0 && ret[re_count].rm_eo >= 0) {
+        pushSTACK(posfixnum(start+ret[re_count].rm_so));
+        pushSTACK(posfixnum(start+ret[re_count].rm_eo));
         funcall(`REGEXP::MAKE-MATCH-BOA`,2); pushSTACK(value1);
       } else pushSTACK(NIL);
-    funcall(L(values),re->re_nsub+1);
+    switch (rettype) {
+      case ret_values:
+        if (re_count < fixnum_to_V(Symbol_value(S(multiple_values_limit)))) {
+          STACK_to_mv(re_count);
+          break;
+        } /* else FALLTHROUGH */
+      case ret_list:
+        VALUES1(listof(re_count));
+        break;
+      case ret_vector:
+        VALUES1(vectorof(re_count));
+        break;
+    }
   }
   skipSTACK(2);                 /* drop pattern & string */
 }
