@@ -1,5 +1,5 @@
 ;;; handle the posix functions
-;;; Sam Steingold 1999-2008
+;;; Sam Steingold 1999-2009
 
 (defpackage #:posix
   (:use #:common-lisp #:ext)
@@ -7,13 +7,13 @@
   (:import-from "SYS" sys::process-id)
   (:shadowing-import-from "EXPORTING" #:defstruct)
   (:export
-   #:resolve-host-ipaddr #:bogomips #:loadavg #:mkstemp #:mkdtemp
+   #:resolve-host-ipaddr #:bogomips #:loadavg #:mkstemp #:mkdtemp #+unix #:wait
    #:stream-lock #:with-stream-lock #:duplicate-handle #:copy-file
    #:file-owner #:physical-memory #:stream-options #+unix #:string-time
    #+(or :win32 :cygwin) #:file-properties #+unix #:make-xterm-io-stream
    #:priority #:openlog #:setlogmask #:syslog #:closelog #:process-id #:getppid
    #:getsid #:setsid #:getpgrp #:setpgrp #:setreuid #:setregid #:kill #:sync
-   #:errno #:strerror #:gethostid
+   #:errno #:strerror #:hostid #:domainname #:file-size #:user-shells
    #:erf #:erfc #:j0 #:j1 #:jn #:y0 #:y1 #:yn #:tgamma #:lgamma #:ffs))
 
 (pushnew :syscalls *features*)
@@ -31,12 +31,12 @@
 ;;; ============================================================
 #+unix (progn
 (export '(uid gid euid egid pgid groups))
-(defsetf uid posix::%setuid)
-(defsetf gid posix::%setgid)
-(defsetf euid posix::%seteuid)
-(defsetf egid posix::%setegid)
-(defsetf pgid posix::%setpgid)
-(defsetf groups posix::%setgroups)
+(defsetf uid %setuid)
+(defsetf gid %setgid)
+(defsetf euid %seteuid)
+(defsetf egid %setegid)
+(defsetf pgid %setpgid)
+(defsetf groups %setgroups)
 )
 ;;; ============================================================
 (defmacro with-stream-lock ((stream &rest options) &body body)
@@ -44,6 +44,10 @@
   `(unwind-protect (progn (stream-lock ,stream t ,@options) ,@body)
      (stream-lock ,stream nil ,@options)))
 ;;; ============================================================
+(defsetf file-size %set-file-size)
+(defsetf hostid %sethostid)
+(defsetf domainname %setdomainname)
+;;; ============================================================
 (defun syslog (severity facility format &rest args)
   (%syslog severity facility (apply #'format nil format args)))
 ;; A compiler macro here is better than a simple (declaim (inline syslog))
@@ -395,9 +399,9 @@
 #+FFI (progn
 (export '(fopen fdopen freopen fclose fflush ; fgetc fputc ungetc
           clearerr feof ferror fileno stdin stdout stderr))
-(defconstant stdin (posix::%stdio 0))
-(defconstant stdout (posix::%stdio 1))
-(defconstant stderr (posix::%stdio 2))
+(defconstant stdin (%stdio 0))
+(defconstant stdout (%stdio 1))
+(defconstant stderr (%stdio 2))
 )
 ;;;--------------------------------------------------------------------------
 (defun ffs (n) (integer-length (logand n (- n))))
