@@ -87,12 +87,7 @@
 ;; DEFINE-CONDITION, CLtL2 p. 898
 (defmacro define-condition (&whole whole-form
                             name parent-types slot-specs &rest options)
-  (unless (symbolp name)
-    (error-of-type 'source-program-error
-      :form whole-form
-      :detail name
-      (TEXT "~S: the name of a condition must be a symbol, not ~S")
-      'define-condition name))
+  (setq name (check-not-declaration name 'define-condition))
   (unless (and (listp parent-types) (every #'symbolp parent-types))
     (error-of-type 'source-program-error
       :form whole-form
@@ -1345,15 +1340,26 @@
 (defun check-value (place condition)
   (let ((restarts
          (nconc
-          (list (make-restart
-                 :name 'USE-VALUE
-                 :report
-                   (lambda (stream)
-                     (format stream (report-one-new-value-string-instead)
-                             place))
-                 :interactive (lambda () (prompt-for-new-value place 1 t))
-                 :invoke-function
-                   (lambda (val) (return-from check-value (values val nil)))))
+          (when (eq place 'SYSTEM::PATHNAME-ENCODING)
+            (setq place '*pathname-encoding*) ; make it look nicer
+            (list (make-restart ; for direntry_to_string
+                   :name 'CONTINUE
+                   :report (lambda (stream)
+                             (format stream
+                                     (TEXT "Discard this directory entry")))
+                   :invoke-function
+                     (lambda () (return-from check-value (values nil 0))))))
+          (unless (eq place '*pathname-encoding*)
+            (list (make-restart
+                   :name 'USE-VALUE
+                   :report
+                     (lambda (stream)
+                       (format stream (report-one-new-value-string-instead)
+                               place))
+                     :interactive (lambda () (prompt-for-new-value place 1 t))
+                     :invoke-function
+                       (lambda (val)
+                         (return-from check-value (values val nil))))))
           (when (and (consp place) (eq 'fdefinition (car place)))
             (list (make-restart ; for check_fdefinition() only!
                    :name 'RETRY
@@ -1403,7 +1409,7 @@
       :report (lambda (out)
                 (format out (TEXT "skip "))
                 (if (compiled-function-p obj)
-                  (write (sys::closure-name obj) :stream out
+                  (write (closure-name obj) :stream out
                          :pretty nil :escape nil)
                   (write obj :stream out :pretty nil :escape nil
                          :level 2 :length 3)))
@@ -1820,3 +1826,42 @@ Returns the added or removed method(s)."
     `(let ((,handlers (set-global-handler nil nil)))
        (unwind-protect (progn ,@body)
          (set-global-handler ,handlers nil)))))
+
+;;; <http://www.lisp.org/HyperSpec/Body/dec_type.html>:
+;;;   A symbol cannot be both the name of a type and the name of a
+;;;   declaration. Defining a symbol as the name of a class, structure,
+;;;   condition, or type, when the symbol has been declared as a
+;;;   declaration name, or vice versa, signals an error.
+(defun check-not-type (symbol caller)
+  (loop
+    (setq symbol (check-symbol symbol caller))
+    (when (handler-bind ((error #'(lambda (c)
+                                    (declare (ignore c))
+                                    (return-from check-not-type symbol))))
+            (type-expand symbol))
+      (with-restarts ((use-value (new-value)
+                        :report
+                         (lambda (stream)
+                           (format stream (report-one-new-value-string-instead)
+                                   symbol))
+                         :interactive
+                          (lambda () (prompt-for-new-value symbol 1 t))
+                         (setq symbol new-value)))
+        (error (TEXT "~S: ~S defines a type, cannot be declared a ~S")
+               caller symbol 'declaration)))))
+
+(defun check-not-declaration (symbol caller)
+  (loop
+    (setq symbol (check-symbol symbol caller))
+    (unless (memq symbol (cdar *toplevel-denv*))
+      (return-from check-not-declaration symbol))
+    (with-restarts ((use-value (new-value)
+                      :report
+                       (lambda (stream)
+                         (format stream (report-one-new-value-string-instead)
+                                 symbol))
+                       :interactive
+                        (lambda () (prompt-for-new-value symbol 1 t))
+                       (setq symbol new-value)))
+      (error (TEXT "~S: ~S names a ~S, cannot name a type")
+             caller symbol 'declaration))))
